package ACL2::Orden::Certifybook;
use base ACL2::Orden;

#
#    Orden CERTIFY-BOOK
#    Copyright (C) 2006-2008 Antonio García Domínguez
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use strict;
use warnings;
use utf8;

use ACL2::Tags;
use ACL2::Localizacion;
use Carp;

my $lh = ACL2::Localizacion->get_handle;

=head2 get_nombre_libro($this)

Uso: ME<eacute>todo pE<uacute>blico.

Responsabilidades: Devuelve el nombre corto (ruta no completa) del
libro incluido.

Entrada: Ninguna.

Salida: Cadena con el nombre corto del libro incluido en la orden.

=cut

sub get_nombre_libro
{
    return $_[0]->{nombre};
}

sub protegido_inicializar_sexpr
{
    my ($this) = @_;

    $this->{nombre} =  $this->get_elems_formula->[1];
    $this->{nombre} =~ s/"//g;

    $this->{k}            = $this->get_elems_formula->[2] || 0;
    $this->{compile_flag} = $this->get_elems_formula->[3] || "T";
}

sub protegido_inicializar_salida
{
    my ($this) = @_;

    # No guardaremos el nombre presente en la S-expresión original -
    # ACL2 genera el nombre absoluto por su cuenta
    my $nombreCorto      = $this->get_elems_formula->[1];
    my $salida = ${$this->get_salida};

    # Dividimos en pasos
    my ($intro, $calculoChecksum, $prueba,
        $inclusion, $escrCert, $compilacion) =
            split /\s*\* Step [0-9]:\s*/, $salida;

    # Descartamos la salida completa
    $this->set_salida("");

    # Copiamos las salidas que nos interesan, según pasos (la del paso
    # 2 se consigue después; la del paso 3 no interesa por sí sola)
    $this->{salida_pasos} = [$intro, $calculoChecksum, undef,
                              undef, $escrCert, $compilacion];

    # INTRODUCCIÓN: Obtención de nombre completo de libro
    ($this->{ruta_libro}) = $intro =~ /CERTIFICATION ATTEMPT FOR "(.*)"/;

    # PASO 2: Dividimos en órdenes y hacemos las demostraciones
    {
        ($this->{salida_pasos}->[2], my @subordenes) =
            split /(\w*?)\s*>>\s*(.*)$/m, $prueba;

        # Repartimos los elementos debidamente
        my @paquetes = ();
        my @ordenes  = ();
        my @salidas  = ();

        # Ahora vamos añadiendo las subórdenes
        $this->{subordenes} = [];
        while (@subordenes > 0) {
            my ($paquete, $sexpr, $salida) = splice @subordenes, 0, 3;
            push @{$this->{subordenes}}, ACL2::Orden::Fabrica->crear($sexpr, $salida, $paquete);
        }
    }

    # Paso 3 - ACL2 realiza una orden INCLUDE-BOOK con el fichero probado
    $this->{inclusion} = ACL2::Orden::Fabrica->crear("(include-book $nombreCorto)",
                                                       $inclusion, $this->get_paquete);

}

sub protegido_cabecera
{
    my ($this, $writer) = @_;

    $writer->startTag(ACL2::Tags::CERTIFYBOOK_TAG,
                      "name" => $this->{nombre},
                      "path" => File::Spec->abs2rel($this->{ruta_libro}),
                      "k"           => $this->{k},
                      "compileFlag" => $this->{compile_flag});
}

sub protegido_cuerpo
{
    my ($this, $writer) = @_;

    # PASO 1
    $writer->startTag(ACL2::Tags::CHECKSUM_TAG);
    ACL2::Orden->salida_parrafos($writer, $this->{salida_pasos}->[1]);
    $writer->endTag(ACL2::Tags::CHECKSUM_TAG);

    # PASO 2
    $writer->startTag(ACL2::Tags::VERIFICATION_TAG);
    ACL2::Orden->salida_parrafos($writer, $this->{salida_pasos}->[2]);
    foreach my $orden (@{$this->{subordenes}}) {
        $orden->escribir_xml($writer);
    }
    $writer->endTag(ACL2::Tags::VERIFICATION_TAG);

    # PASO 3
    $writer->startTag(ACL2::Tags::INCLUSIONCHECK_TAG);
    $this->{inclusion}->escribir_xml($writer);
    $writer->endTag(ACL2::Tags::INCLUSIONCHECK_TAG);

    # PASO 4
    $writer->startTag(ACL2::Tags::WRITECERT_TAG);
    ACL2::Orden->salida_parrafos($writer, $this->{salida_pasos}->[4]);
    $writer->endTag(ACL2::Tags::WRITECERT_TAG);

    # PASO 5
    $writer->startTag(ACL2::Tags::COMPILE_TAG);
    ACL2::Orden->salida_parrafos($writer, $this->{salida_pasos}->[5]);
    $writer->endTag(ACL2::Tags::COMPILE_TAG);
}

1;
